# -*- coding: utf-8 -*-
import textwrap
from collections import OrderedDict
from decimal import Decimal

from _stories.compat import indent
from _stories.exceptions import MutationError


ATTRIBUTE_ERROR_MSG = textwrap.dedent(
    """
    '{obj}' object has no attribute {attr}

    {ctx!r}
    """
).strip()


def make_context(contract, kwargs, history):
    kwargs = contract.check_story_call(kwargs)
    ns = OrderedDict(
        # FIXME: We should be able to remove `if` statement here.
        (arg, kwargs[arg])
        for arg in sorted(contract.argset)
        if arg in kwargs
    )
    ctx = Context()
    ctx.__dict__["_Context__ns"] = ns
    ctx.__dict__["_Context__history"] = history
    ctx.__dict__["_Context__lines"] = ["Story argument"] * len(ns)
    return ctx


class Context(object):
    def __getattr__(self, name):
        try:
            return self.__ns[name]
        except KeyError:
            raise AttributeError(
                ATTRIBUTE_ERROR_MSG.format(obj="Context", attr=name, ctx=self)
            )

    def __setattr__(self, name, value):
        raise MutationError(assign_attribute_message)

    def __delattr__(self, name):
        raise MutationError(delete_attribute_message)

    def __repr__(self):
        return history_representation(self) + "\n\n" + context_representation(self)

    def __dir__(self):
        spec = type("Context", (object,), {})
        parent = set(dir(spec()))
        current = set(self.__dict__) - {
            "_Context__ns",
            "_Context__history",
            "_Context__lines",
        }
        scope = set(self.__ns)
        attributes = sorted(parent | current | scope)
        return attributes

    def __bool__(self):
        message = comparison_template.format(available=", ".join(map(repr, self.__ns)))
        raise MutationError(message)

    __nonzero__ = __bool__


def assign_namespace(ctx, method, kwargs):
    ctx._Context__ns.update((arg, kwargs[arg]) for arg in sorted(kwargs))
    line = "Set by {}.{}".format(method.__self__.__class__.__name__, method.__name__)
    ctx._Context__lines.extend([line] * len(kwargs))


def history_representation(ctx):
    result = "\n".join(ctx._Context__history.lines)
    return result


def context_representation(ctx, repr_func=repr):
    if not ctx._Context__lines:
        return "Context()"
    seen = []
    items = []
    longest = 0
    for key, value in ctx._Context__ns.items():
        for seen_key, seen_value in seen:
            if value is seen_value:
                item = "`{}` alias".format(seen_key)
                break
        else:
            item = repr_func(value)
        too_long = len(key) + len(item) + 4 > 88
        has_new_lines = "\n" in item
        if too_long or has_new_lines:
            head = key + ":"
            tail = "\n" + indent(item, "    ")
        else:
            head = "{}: {}".format(key, item)
            tail = ""
        if type(value) not in [type(None), bool, int, float, Decimal]:
            seen.append((key, value))
        items.append((head, tail))
        head_length = len(head)
        if head_length > longest:
            longest = head_length
    lines = [
        "  {}  # {}{}".format(head.ljust(longest), line, tail)
        for (head, tail), line in zip(items, ctx._Context__lines)
    ]
    return "\n".join(["Context:"] + lines)


# Messages.


assign_attribute_message = """
Context object is immutable.

Use Success() keyword arguments to expand its scope.
""".strip()


delete_attribute_message = """
Context object is immutable.

Variables can not be removed from Context.
""".strip()


comparison_template = """
Context object can not be used in boolean comparison.

Available variables: {available}
""".strip()



# -*- coding: utf-8 -*-


class FailureSummary(object):
    def __init__(self, protocol, ctx, failed_method, reason):
        self.__protocol = protocol
        self.is_success = False
        self.is_failure = True
        self.ctx = ctx
        self.__failed_method = failed_method
        self.__failure_reason = reason

    def failed_on(self, method_name):
        return method_name == self.__failed_method

    def failed_because(self, reason):
        self.__protocol.check_failed_because_argument(reason)
        return self.__protocol.compare_failed_because_argument(
            reason, self.__failure_reason
        )

    @property
    def value(self):
        raise AssertionError

    def __repr__(self):
        return "Failure()"


class SuccessSummary(object):
    def __init__(self, protocol, value):
        self.__protocol = protocol
        self.is_success = True
        self.is_failure = False
        self.value = value

    def failed_on(self, method_name):
        return False

    def failed_because(self, reason):
        self.__protocol.check_failed_because_argument(reason)
        return False

    def __repr__(self):
        return "Success()"



from typing import Any
from typing import Callable
from typing import List
from typing import overload
from typing import Tuple
from typing import Union

from _stories.context import Context
from _stories.contract import NullContract
from _stories.contract import SpecContract
from _stories.failures import DisabledNullExecProtocol
from _stories.failures import NotNullExecProtocol
from _stories.failures import NullExecProtocol
from _stories.history import History
from _stories.marker import BeginningOfStory
from _stories.marker import EndOfStory
from _stories.returned import Failure
from _stories.returned import Result
from _stories.returned import Skip
from _stories.returned import Success
from _stories.run import Call
from _stories.run import Run
from _stories.summary import FailureSummary
from _stories.summary import SuccessSummary
@overload
def execute(
    runner: Call,
    ctx: Context,
    history: History,
    methods: List[
        Tuple[
            Union[
                BeginningOfStory,
                Callable[[Context], Union[Result, Success, Failure, Skip]],
                EndOfStory,
            ],
            Union[NullContract, SpecContract],
            Union[NullExecProtocol, DisabledNullExecProtocol, NotNullExecProtocol],
        ],
    ],
) -> Any: ...
@overload
def execute(
    runner: Run,
    ctx: Context,
    history: History,
    methods: List[
        Tuple[
            Union[
                BeginningOfStory,
                Callable[[Context], Union[Result, Success, Failure, Skip]],
                EndOfStory,
            ],
            Union[NullContract, SpecContract],
            Union[NullExecProtocol, DisabledNullExecProtocol, NotNullExecProtocol],
        ],
    ],
) -> Union[SuccessSummary, FailureSummary]: ...



from _stories.context import Context
from _stories.returned import Success

class BeginningOfStory:
    def __init__(self, cls_name: str, name: str) -> None: ...
    def __call__(self, ctx: Context) -> Success: ...
    def set_parent(self, parent_name: str, same_object: bool) -> None: ...

class EndOfStory:
    def __call__(self, ctx: Context) -> Success: ...



# -*- coding: utf-8 -*-
from _stories.exceptions import StoryDefinitionError


def arguments(*names):
    if not names:
        raise StoryDefinitionError("Story arguments can not be an empty list")

    if any(name for name in names if not isinstance(name, str)):
        message = "Story arguments can only be defined with string type"
        raise StoryDefinitionError(message)

    def decorator(f):
        f.arguments = list(names)
        return f

    return decorator


def get_arguments(f):
    return getattr(f, "arguments", [])



from enum import Enum
from typing import Any
from typing import Dict
from typing import Optional
from typing import Union

class Result:
    def __init__(self, value: Any = ...) -> None: ...
    def __repr__(self) -> str: ...

class Success:
    def __init__(self, **kwargs: Dict[str, Any]) -> None: ...
    def __repr__(self) -> str: ...

class Failure:
    def __init__(self, reason: Optional[Union[str, Enum]] = ...) -> None: ...
    def __repr__(self) -> str: ...

class Skip:
    def __repr__(self) -> str: ...



# -*- coding: utf-8 -*-
# type: ignore
from debug_toolbar.panels import Panel
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext_lazy as __

import _stories.context


# FIXME: Test me.
#
# FIXME: Type me.


origin_context_init = _stories.context.Context.__init__


def track_context(storage):
    def wrapper(ctx):
        origin_context_init(ctx)
        storage.append(ctx)

    return wrapper


class StoriesPanel(Panel):

    # Implement the Panel API

    template = "stories/debug_toolbar/stories_panel.html"

    nav_title = _("Stories")

    @property
    def nav_subtitle(self):
        count = len(self.storage)
        return __("%(count)d call", "%(count)d calls", count) % {"count": count}

    @property
    def title(self):
        count = len(self.storage)
        return __(
            "Context and execution path of %(count)d story",
            "Context and execution path of %(count)d stories",
            count,
        ) % {"count": count}

    # Implement the Collector.

    def __init__(self, *args, **kwargs):
        super(StoriesPanel, self).__init__(*args, **kwargs)
        self.storage = []

    def enable_instrumentation(self):
        _stories.context.Context.__init__ = track_context(self.storage)

    def disable_instrumentation(self):
        _stories.context.Context.__init__ = origin_context_init

    def generate_stats(self, request, response):
        self.record_stats({"stories": self.storage})



# -*- coding: utf-8 -*-
from _stories.context import make_context
from _stories.execute import function
from _stories.failures import make_run_protocol
from _stories.history import History
from _stories.marker import BeginningOfStory
from _stories.marker import EndOfStory
from _stories.run import Call
from _stories.run import Run


class ClassMountedStory(object):
    def __init__(self, cls, name, collected, contract, failures):
        self.cls = cls
        self.name = name
        self.collected = collected
        self.contract = contract
        self.failures = failures

    def __repr__(self):
        result = [self.cls.__name__ + "." + self.name]
        for name in self.collected:
            attr = getattr(self.cls, name, None)
            if type(attr) is ClassMountedStory:
                result.append("  " + attr.name)
                result.extend(["  " + line for line in repr(attr).splitlines()[1:]])
            else:
                defined = "" if attr else " ??"
                result.append("  " + name + defined)
        return "\n".join(result)


class MountedStory(object):
    def __init__(self, obj, cls_name, name, arguments, methods, contract, failures):
        self.obj = obj
        self.cls_name = cls_name
        self.name = name
        self.arguments = arguments
        self.methods = methods
        self.contract = contract
        self.failures = failures

    def __call__(self, **kwargs):
        __tracebackhide__ = True
        history = History()
        ctx = make_context(self.methods[0][1], kwargs, history)
        runner = Call()
        return function.execute(runner, ctx, history, self.methods)

    def run(self, **kwargs):
        __tracebackhide__ = True
        history = History()
        ctx = make_context(self.methods[0][1], kwargs, history)
        run_protocol = make_run_protocol(self.failures, self.cls_name, self.name)
        runner = Run(run_protocol)
        return function.execute(runner, ctx, history, self.methods)

    def __repr__(self):
        result = []
        indent = 0
        for method, _contract, _protocol in self.methods:
            method_type = type(method)
            if method_type is EndOfStory:
                indent -= 1
            else:
                result.append("  " * indent + method.__name__)
                if method_type is BeginningOfStory:
                    indent += 1
        return "\n".join(result)



# -*- coding: utf-8 -*-
from _stories.compat import Enum
from _stories.compat import EnumMeta
from _stories.exceptions import FailureProtocolError


# Data type.


def check_data_type(failures):
    if failures is None:
        return
    if isinstance(failures, EnumMeta):
        return
    if isinstance(failures, (list, tuple, set, frozenset)) and all(
        isinstance(failure, str) for failure in failures
    ):
        return
    message = wrong_type_template.format(failures=failures)
    raise FailureProtocolError(message)


def failures_representation(failures):
    if isinstance(failures, EnumMeta):
        return ", ".join(map(repr, failures.__members__.values()))
    elif isinstance(failures, (list, tuple, set, frozenset)):
        return ", ".join(map(repr, failures))
    elif failures is None:
        return "None"


def collection_contains(reason, failures):
    return reason in failures


def collection_compare(a, b):
    return a == b


def enumeration_contains(reason, failures):
    return isinstance(reason, Enum) and reason.name in failures.__members__


def enumeration_compare(a, b):
    return a.name == b.name


# Execute.


def make_exec_protocol(failures):

    if isinstance(failures, EnumMeta):
        return NotNullExecProtocol(failures, enumeration_contains)
    elif isinstance(failures, (list, tuple, set, frozenset)):
        return NotNullExecProtocol(failures, collection_contains)
    elif failures is None:
        return NullExecProtocol()


class NullExecProtocol(object):
    def check_return_statement(self, method, reason):
        if reason:
            message = null_protocol_template.format(
                reason=reason,
                cls=method.__self__.__class__.__name__,
                method=method.__name__,
            )
            raise FailureProtocolError(message)


class DisabledNullExecProtocol(NullExecProtocol):
    def check_return_statement(self, method, reason):
        if not reason:
            message = disabled_null_template.format(
                cls=method.__self__.__class__.__name__, method=method.__name__
            )
            raise FailureProtocolError(message)
        super(DisabledNullExecProtocol, self).check_return_statement(method, reason)


class NotNullExecProtocol(object):
    def __init__(self, failures, contains_func):
        self.failures = failures
        self.contains_func = contains_func

    def check_return_statement(self, method, reason):
        if not reason:
            message = null_reason_template.format(
                available=failures_representation(self.failures),
                cls=method.__self__.__class__.__name__,
                method=method.__name__,
            )
            raise FailureProtocolError(message)
        if not self.contains_func(reason, self.failures):
            message = wrong_reason_template.format(
                reason=reason,
                available=failures_representation(self.failures),
                cls=method.__self__.__class__.__name__,
                method=method.__name__,
            )
            raise FailureProtocolError(message)


# Run.


def make_run_protocol(failures, cls_name, method_name):

    if isinstance(failures, EnumMeta):
        return NotNullRunProtocol(
            cls_name, method_name, failures, enumeration_contains, enumeration_compare
        )
    elif isinstance(failures, (list, tuple, set, frozenset)):
        return NotNullRunProtocol(
            cls_name, method_name, failures, collection_contains, collection_compare
        )
    elif failures is None:
        return NullRunProtocol(cls_name, method_name)


class NullRunProtocol(object):
    def __init__(self, cls_name, method_name):
        self.cls_name = cls_name
        self.method_name = method_name

    def check_failed_because_argument(self, reason):
        message = null_summary_template.format(
            cls=self.cls_name, method=self.method_name
        )
        raise FailureProtocolError(message)


class NotNullRunProtocol(object):
    def __init__(self, cls_name, method_name, failures, contains_func, compare_func):
        self.cls_name = cls_name
        self.method_name = method_name
        self.failures = failures
        self.contains_func = contains_func
        self.compare_func = compare_func

    def check_failed_because_argument(self, reason):
        if not self.contains_func(reason, self.failures):
            message = wrong_summary_template.format(
                reason=reason,
                available=failures_representation(self.failures),
                cls=self.cls_name,
                method=self.method_name,
            )
            raise FailureProtocolError(message)

    def compare_failed_because_argument(self, argument, failure_reason):
        return self.compare_func(argument, failure_reason)


# Wrap.


def combine_failures(
    first_failures,
    first_cls_name,
    first_method_name,
    second_failures,
    second_cls_name,
    second_method_name,
):
    if first_failures is None:
        return second_failures
    elif second_failures is None:
        return first_failures
    elif isinstance(first_failures, EnumMeta) and isinstance(second_failures, EnumMeta):
        return Enum(
            first_failures.__name__,
            ",".join(
                list(first_failures.__members__.keys())
                + [
                    failure
                    for failure in second_failures.__members__.keys()
                    if failure not in first_failures.__members__.keys()
                ]
            ),
        )
    elif isinstance(first_failures, (list, tuple, set, frozenset)) and isinstance(
        second_failures, (list, tuple, set, frozenset)
    ):
        return first_failures + [
            failure for failure in second_failures if failure not in first_failures
        ]
    else:
        message = type_error_template.format(
            cls=first_cls_name,
            method=first_method_name,
            available=failures_representation(first_failures),
            other_cls=second_cls_name,
            other_method=second_method_name,
            other_available=failures_representation(second_failures),
        )
        raise FailureProtocolError(message)


def maybe_disable_null_protocol(methods, reasons):

    if reasons is None:
        return methods
    disabled = DisabledNullExecProtocol()
    return [
        (method, contract, disabled if type(protocol) is NullExecProtocol else protocol)
        for method, contract, protocol in methods
    ]


# Messages.


wrong_type_template = """
Unexpected type for story failure protocol: {failures!r}
""".strip()


wrong_reason_template = """
Failure({reason!r}) failure reason is not allowed by current protocol.

Available failures are: {available}

Function returned value: {cls}.{method}
""".strip()


null_reason_template = """
Failure() can not be used in a story with failure protocol.

Available failures are: {available}

Function returned value: {cls}.{method}

Use one of them as Failure() argument.
""".strip()


null_protocol_template = """
Failure({reason!r}) can not be used in a story without failure protocol.

Function returned value: {cls}.{method}

Use 'failures' story method to define failure protocol.
""".strip()


wrong_summary_template = """
'failed_because' method got argument mismatching failure protocol: {reason!r}

Available failures are: {available}

Story returned result: {cls}.{method}
""".strip()


null_summary_template = """
'failed_because' method can not be used with story defined without failure protocol.

Story returned result: {cls}.{method}

Use 'failures' story method to define failure protocol.
""".strip()


type_error_template = """
Story and substory failure protocols has incompatible types:

Story method: {cls}.{method}

Story failure protocol: {available}

Substory method: {other_cls}.{other_method}

Substory failure protocol: {other_available}
""".strip()


disabled_null_template = """
Failure() can not be used in a story composition.

Different types of failure protocol were used in parent and substory definitions.

Function returned value: {cls}.{method}

Use 'failures' story method to define failure protocol.
""".strip()



from enum import Enum
from typing import Any
from typing import NoReturn
from typing import Optional
from typing import Union

from _stories.context import Context
from _stories.failures import NotNullRunProtocol
from _stories.failures import NullRunProtocol

class FailureSummary:
    def __init__(
        self,
        protocol: Union[NullRunProtocol, NotNullRunProtocol],
        ctx: Context,
        failed_method: str,
        reason: Optional[Union[str, Enum]],
    ) -> None: ...
    def failed_on(self, method_name: str) -> bool: ...
    def failed_because(self, reason: Union[str, Enum]) -> bool: ...
    @property
    def value(self) -> NoReturn: ...
    def __repr__(self) -> str: ...

class SuccessSummary:
    def __init__(
        self, protocol: Union[NullRunProtocol, NotNullRunProtocol], value: Any
    ) -> None: ...
    def failed_on(self, method_name: str) -> bool: ...
    def failed_because(self, reason: str) -> bool: ...
    def __repr__(self) -> str: ...



# -*- coding: utf-8 -*-
# type: ignore
import linecache
import sys
import textwrap

from _pytest.config import hookimpl

import _stories.compat
import _stories.context


# FIXME: Test me.
#
# FIXME: Type me.


origin_context_init = _stories.context.Context.__init__


def track_context(storage):
    def wrapper(ctx):
        origin_context_init(ctx)
        storage.append((get_test_source(*get_test_call()), ctx))

    return wrapper


def get_test_call():
    f = sys._getframe()

    while True:
        if (
            "@py_builtins" in f.f_globals
            and "@pytest_ar" in f.f_globals
            and f.f_code.co_filename != __file__
        ):
            return f.f_code.co_filename, f.f_lineno
        elif not f.f_back:
            raise Exception("Can not find running test")
        else:
            f = f.f_back


def get_test_source(filename, lineno):

    start = max(1, lineno - 3)
    end = lineno + 3
    adjust_to = len(str(end))

    lines = [linecache.getline(filename, no) for no in range(start, end)]
    text = textwrap.dedent("".join(lines))

    src = []
    for num, line in zip(range(start, end), text.splitlines()):
        sep = "->" if num == lineno else "  "
        src.append((" {} {} {}".format(str(num).rjust(adjust_to), sep, line)).rstrip())

    src = "\n".join(src)

    return src


@hookimpl(hookwrapper=True)
def pytest_runtest_call(item):
    storage = []
    _stories.context.Context.__init__ = track_context(storage)
    yield
    _stories.context.Context.__init__ = origin_context_init
    for i, (src, ctx) in enumerate(storage, 1):
        output = "\n\n".join(
            [
                src,
                _stories.context.history_representation(ctx)
                + "\n\n"
                + _stories.context.context_representation(ctx, _stories.compat.pformat),
            ]
        )
        item.add_report_section("call", "story #%d" % (i,), output)



from enum import Enum
from typing import Optional
from typing import Union

class StoryError: ...
class StoryDefinitionError: ...

class FailureError:
    def __init__(self, reason: Optional[Union[str, Enum]]) -> None: ...
    def __repr__(self) -> str: ...

class FailureProtocolError: ...
class ContextContractError: ...
class MutationError: ...



# -*- coding: utf-8 -*-
# type: ignore
"""
stories.contrib.debug_toolbars.flask
------------------------------------

This module contains integration with flask-debugtoolbar.

:copyright: (c) 2018-2020 dry-python team.
:license: BSD, see LICENSE for more details.
"""
from _stories.contrib.debug_toolbars.flask.panels import StoriesPanel


__all__ = ["StoriesPanel"]



# -*- coding: utf-8 -*-


class History(object):
    def __init__(self):
        self.indent = 0
        self.lines = []

    def before_call(self, method_name):
        self.lines.append("  " * self.indent + method_name)

    def on_result(self, value):
        self.lines[-1] += " (returned: " + repr(value) + ")"

    def on_failure(self, reason):
        if reason:
            self.lines[-1] += " (failed: " + repr(reason) + ")"
        else:
            self.lines[-1] += " (failed)"

    def on_skip(self):
        self.lines[-1] += " (skipped)"
        self.indent -= 1

    def on_error(self, error_name):
        self.lines[-1] += " (errored: " + error_name + ")"

    def on_substory_start(self):
        self.indent += 1

    def on_substory_end(self):
        self.lines.pop()
        self.indent -= 1



# -*- coding: utf-8 -*-
from _stories.exceptions import StoryDefinitionError


def collect_story(f):

    calls = []

    class Collector(object):
        def __getattr__(self, name):
            calls.append(name)

    f(Collector())

    if not calls:
        raise StoryDefinitionError("Story should have at least one step defined")

    return calls



# -*- coding: utf-8 -*-
"""
stories.shortcuts
-----------------

This module contains convenient functions to reduce boilerplate code.

:copyright: (c) 2018-2020 dry-python team.
:license: BSD, see LICENSE for more details.
"""
from _stories.shortcuts import contract_in
from _stories.shortcuts import failures_in


__all__ = ["contract_in", "failures_in"]



from enum import Enum
from typing import Callable
from typing import Dict
from typing import List
from typing import overload
from typing import Tuple
from typing import Type
from typing import TypeVar
from typing import Union

from cerberus.validator import Validator
from marshmallow import Schema
from pydantic.main import BaseModel

_T = TypeVar("_T")
@overload
def contract_in(
    cls: _T, *args: Tuple[()]
) -> Callable[[Union[BaseModel, Schema]], _T]: ...
@overload
def contract_in(cls: _T, *args: Tuple[Dict[str, Callable]]) -> Dict[str, Callable]: ...
@overload
def contract_in(cls: _T, *args: Tuple[Validator]) -> Validator: ...
@overload
def failures_in(cls: _T, *args: Tuple[()]) -> Callable[[Type[Enum]], _T]: ...
@overload
def failures_in(cls: _T, *args: Tuple[List[str]]) -> List[str]: ...



from enum import Enum
from typing import Any
from typing import Callable
from typing import List
from typing import Optional
from typing import Type
from typing import Union

from _stories.mounted import ClassMountedStory
from _stories.mounted import MountedStory

class Story:
    def __init__(self, f: Callable) -> None: ...
    def __get__(self, obj: Any, cls: Any) -> Union[MountedStory, ClassMountedStory]: ...
    def contract(self, contract: Any) -> Any: ...
    def failures(self, failures: Any) -> Optional[Union[List[str], Type[Enum]]]: ...



# -*- coding: utf-8 -*-
# type: ignore
"""
stories.contrib.pytest
----------------------

This module contains integration with PyTest framework.

:copyright: (c) 2018-2020 dry-python team.
:license: BSD, see LICENSE for more details.
"""
from _stories.contrib.pytest import pytest_runtest_call


__all__ = ["pytest_runtest_call"]



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['_stories',
 '_stories.contrib',
 '_stories.contrib.debug_toolbars',
 '_stories.contrib.debug_toolbars.django',
 '_stories.contrib.debug_toolbars.flask',
 '_stories.execute',
 'stories',
 'stories.contrib',
 'stories.contrib.debug_toolbars',
 'stories.contrib.sentry']

package_data = \
{'': ['*'],
 '_stories.contrib.debug_toolbars.django': ['templates/stories/debug_toolbar/*'],
 '_stories.contrib.debug_toolbars.flask': ['templates/stories/debug_toolbar/*']}

extras_require = \
{'mkdocs': ['mkdocs', 'mkdocs-material']}

entry_points = \
{'pytest11': ['stories = stories.contrib.pytest']}

setup_kwargs = {
    'name': 'stories',
    'version': '0.11.1',
    'description': 'Define a user story in the business transaction DSL',
    'long_description': "![logo](https://raw.githubusercontent.com/dry-python/brand/master/logo/stories.png)\n\n[![azure-pipeline](https://dev.azure.com/dry-python/stories/_apis/build/status/dry-python.stories?branchName=master)](https://dev.azure.com/dry-python/stories/_build/latest?definitionId=3&branchName=master)\n[![codecov](https://codecov.io/gh/dry-python/stories/branch/master/graph/badge.svg)](https://codecov.io/gh/dry-python/stories)\n[![docs](https://readthedocs.org/projects/stories/badge/?version=latest)](https://stories.readthedocs.io/en/latest/?badge=latest)\n[![gitter](https://badges.gitter.im/dry-python/stories.svg)](https://gitter.im/dry-python/stories)\n[![pypi](https://img.shields.io/pypi/v/stories.svg)](https://pypi.python.org/pypi/stories/)\n[![black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/ambv/black)\n\n---\n\n# The business transaction DSL\n\n- [Source Code](https://github.com/dry-python/stories)\n- [Issue Tracker](https://github.com/dry-python/stories/issues)\n- [Documentation](https://stories.readthedocs.io/en/latest/)\n- [Discussion](https://gitter.im/dry-python/stories)\n\n## Installation\n\nAll released versions are hosted on the Python Package Index. You can\ninstall this package with following command.\n\n```bash\npip install stories\n```\n\n## Usage\n\n`stories` provide a simple way to define a complex business scenario\nthat include many processing steps.\n\n```pycon\n\n>>> from stories import story, arguments, Success, Failure, Result\n>>> from django_project.models import Category, Profile, Subscription\n\n>>> class Subscribe:\n...\n...     @story\n...     @arguments('category_id', 'profile_id')\n...     def buy(I):\n...\n...         I.find_category\n...         I.find_profile\n...         I.check_balance\n...         I.persist_subscription\n...         I.show_subscription\n...\n...     def find_category(self, ctx):\n...\n...         category = Category.objects.get(pk=ctx.category_id)\n...         return Success(category=category)\n...\n...     def find_profile(self, ctx):\n...\n...         profile = Profile.objects.get(pk=ctx.profile_id)\n...         return Success(profile=profile)\n...\n...     def check_balance(self, ctx):\n...\n...         if ctx.category.cost < ctx.profile.balance:\n...             return Success()\n...         else:\n...             return Failure()\n...\n...     def persist_subscription(self, ctx):\n...\n...         subscription = Subscription(category=ctx.category, profile=ctx.profile)\n...         subscription.save()\n...         return Success(subscription=subscription)\n...\n...     def show_subscription(self, ctx):\n...\n...         return Result(ctx.subscription)\n\n```\n\n```pycon\n\n>>> Subscribe().buy(category_id=1, profile_id=1)\n<Subscription: Subscription object (8)>\n\n```\n\nThis code style allow you clearly separate actual business scenario from\nimplementation details.\n\n## License\n\nStories library is offered under the two clause BSD license.\n",
    'author': 'Artem Malyshev',
    'author_email': 'proofit404@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://dry-python.org/',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*',
}


setup(**setup_kwargs)



from enum import Enum
from typing import List
from typing import NoReturn
from typing import Optional
from typing import Union

from _stories.context import Context
from _stories.failures import NotNullRunProtocol
from _stories.failures import NullRunProtocol
from _stories.summary import FailureSummary
from _stories.summary import SuccessSummary

class Call:
    def got_failure(
        self, ctx: Context, method_name: str, reason: Optional[Union[str, Enum]]
    ) -> NoReturn: ...
    def got_result(self, value: Union[List[str], int]) -> Union[List[str], int]: ...
    def finished(self) -> None: ...

class Run:
    def __init__(
        self, protocol: Union[NotNullRunProtocol, NullRunProtocol]
    ) -> None: ...
    def got_failure(
        self, ctx: Context, method_name: str, reason: Optional[Union[str, Enum]]
    ) -> FailureSummary: ...
    def got_result(self, value: Union[List[str], int]) -> SuccessSummary: ...
    def finished(self) -> SuccessSummary: ...



from typing import Any
from typing import Callable
from typing import List
from typing import Tuple
from typing import Union

from _stories.contract import NullContract
from _stories.contract import SpecContract
from _stories.failures import NotNullExecProtocol
from _stories.failures import NullExecProtocol
from _stories.marker import BeginningOfStory
from _stories.marker import EndOfStory

def wrap_story(
    arguments: List[str],
    collected: List[str],
    cls_name: str,
    story_name: str,
    obj: Any,
    spec: None,
    failures: None,
) -> Tuple[
    List[
        Tuple[
            Union[BeginningOfStory, Callable, EndOfStory],
            Union[NullContract, SpecContract],
            Union[NullExecProtocol, NotNullExecProtocol],
        ],
    ],
    NullContract,
    None,
]: ...



# -*- coding: utf-8 -*-
"""
stories.exceptions
------------------

This module contains errors definitions user can handle.

:copyright: (c) 2018-2020 dry-python team.
:license: BSD, see LICENSE for more details.
"""
from _stories.exceptions import ContextContractError
from _stories.exceptions import FailureError
from _stories.exceptions import FailureProtocolError
from _stories.exceptions import MutationError
from _stories.exceptions import StoryDefinitionError
from _stories.exceptions import StoryError


__all__ = [
    "ContextContractError",
    "FailureError",
    "FailureProtocolError",
    "MutationError",
    "StoryDefinitionError",
    "StoryError",
]
